// shifter screen saver 
// by douglas irving repetto
// based on code from be inc.
//
// shifts screen image by 15 bytes at a time, skewing colors

#include <add-ons/screen_saver/ScreenSaver.h> 
#include <interface/StringView.h> 
#include <Bitmap.h>
#include <BeBuild.h> 
#include <stdlib.h>

extern "C" { 
_EXPORT BScreenSaver *instantiate_screen_saver( BMessage *msg, image_id id ); 
} 

rgb_color	black = {0,0,0};
rgb_color	white = {255,255,255};

class MinimalSaver : public BScreenSaver 
{ 
public: 
    MinimalSaver( BMessage *archive, image_id image ); 
    virtual ~MinimalSaver( void ); 
    
    virtual void StartConfig( BView *view ); 
    virtual status_t StartSaver( BView *view, bool preview ); 
	virtual void StopSaver();
    virtual void Draw( BView *view, int32 frame ); 
   	void GetPixel(BBitmap *bitmap,const BPoint& pt,rgb_color& color);
    
    rgb_color	colorBase1;
	rgb_color	colorBase2;
	rgb_color	colorCurr1;
	rgb_color	colorCurr2;
	bool		doingPreview;
	BBitmap* mScreenBitmap;
	uint8* rowBytes;
	int32 bytesPerRow;
}; 

BScreenSaver *instantiate_screen_saver( BMessage *msg, image_id id ) 
{ 
    return new MinimalSaver( msg, id ); 
} 

MinimalSaver::MinimalSaver( BMessage *archive, image_id image ) 
    : BScreenSaver( archive, image ) 
{     
	mScreenBitmap = 0;
    SetTickSize(0);
} 

MinimalSaver::~MinimalSaver( void ) 
{ 
	delete mScreenBitmap;
    // Nothing else to do here... 
} 

void MinimalSaver::StartConfig( BView *view ) 
{ 
    view->AddChild( new BStringView( BRect( 10, 10, 200, 35 ), 
                                     B_EMPTY_STRING, 
                                     "shifter screensaver, by d.i.r." ) ); 
} 

status_t MinimalSaver::StartSaver( BView *view, bool preview ) 
{ 
	BRect screenFrame;
	
    if (preview)
    	doingPreview = true;
    else
    	doingPreview = false;

	srandom(system_time());
	
	BScreen screen(B_MAIN_SCREEN_ID);
	screenFrame = screen.Frame();
	screenFrame.PrintToStream();

	/* a 32-bit screen bitmap is obviously large and perhaps
	** wasteful when the actual screen bit depth is lower, but
	** it makes grabbing the pixel values in GetPixel easy
	*/

	mScreenBitmap = new BBitmap(screenFrame,B_RGB32,false);
	if (mScreenBitmap->IsValid()) {
		// perhaps BScreen::GetBitmap is the ticket now? Investigate...
		//_get_screen_bitmap_(mScreenBitmap,screenFrame,false);
		screen.ReadBitmap(mScreenBitmap);
	}
	else {
		return (B_ERROR);
	}

	bytesPerRow = mScreenBitmap->BytesPerRow();
	rowBytes = new uint8(bytesPerRow);
    return B_OK; 
} 

void MinimalSaver::StopSaver()
{
	delete(rowBytes);
}

void MinimalSaver::Draw( BView *view, int32 frame ) 
{ 
	BRect rect = view->Bounds();
	BRect currRect;
	int32 offset;
	static int32 whichRow = 1;
	static int32 previewCount = 0;
	
	offset = 15;
	currRect = BRect(0.0, whichRow, rect.right, whichRow + 1);
	
	//if we're in the preview, only draw every 20 frames...
	if (!doingPreview || previewCount % 10 == 0) {
		if (view->Window()->Lock()) {

			memcpy(((uint8 *)mScreenBitmap->Bits() + (whichRow * bytesPerRow)), 
				((uint8 *)mScreenBitmap->Bits() + (whichRow * bytesPerRow)) + offset,
				bytesPerRow);		
			
			view->DrawBitmap(mScreenBitmap, currRect, currRect);
			view->Window()->Unlock();
			
			whichRow += 1;
			if (whichRow >= (int32)rect.Height() - 1)
				whichRow = 1;
		}
	}
	previewCount++;
}